@@ -1,4 +1,4 @@
-# Copyright 2016-2020 The Van Valen Lab at the California Institute of
+# Copyright 2016-2021 The Van Valen Lab at the California Institute of
 # Technology (Caltech), with support from the Paul Allen Family Foundation,
 # Google, & National Institutes of Health (NIH) under Grant U24CA224309-01.
 # All rights reserved.
@@ -31,17 +31,17 @@ from __future__ import print_function
 
 import os
 
-from tensorflow.keras.utils import get_file
+import tensorflow as tf
 
+from deepcell_toolbox.processing import histogram_normalization
 from deepcell_toolbox.deep_watershed import deep_watershed
-from deepcell_toolbox.processing import phase_preprocess
 
 from deepcell.applications import Application
-from deepcell.model_zoo import PanopticNet
 
 
-WEIGHTS_PATH = ('https://deepcell-data.s3-us-west-1.amazonaws.com/'
-                'model-weights/general_cyto_named.h5')
+MODEL_PATH = ('https://deepcell-data.s3-us-west-1.amazonaws.com/'
+              'saved-models/CytoplasmSegmentation-3.tar.gz')
+MODEL_HASH = '6a244f561b4d37169cb1a58b6029910f'
 
 
 class CytoplasmSegmentation(Application):
@@ -84,48 +84,34 @@ class CytoplasmSegmentation(Application):
 
     #: Metadata for the model and training process
     model_metadata = {
-        'batch_size': 2,
+        'batch_size': 16,
         'lr': 1e-4,
-        'lr_decay': 0.95,
+        'lr_decay': 0.9,
         'training_seed': 0,
         'n_epochs': 8,
         'training_steps_per_epoch': 7899 // 2,
         'validation_steps_per_epoch': 1973 // 2
     }
 
-    def __init__(self,
-                 use_pretrained_weights=True,
-                 model_image_shape=(128, 128, 1)):
+    def __init__(self, model=None,
+                 preprocessing_fn=histogram_normalization,
+                 postprocessing_fn=deep_watershed):
 
-        model = PanopticNet('resnet50',
-                            input_shape=model_image_shape,
-                            norm_method='whole_image',
-                            num_semantic_heads=2,
-                            num_semantic_classes=[1, 1],
-                            location=True,
-                            include_top=True,
-                            lite=True,
-                            use_imagenet=use_pretrained_weights,
-                            interpolation='bilinear')
-
-        if use_pretrained_weights:
-            weights_path = get_file(
-                os.path.basename(WEIGHTS_PATH),
-                WEIGHTS_PATH,
-                cache_subdir='models',
-                file_hash='104a7d7884c80c37d2bce6d1c3a17c7a'
+        if model is None:
+            archive_path = tf.keras.utils.get_file(
+                'CytoplasmSegmentation.tgz', MODEL_PATH,
+                file_hash=MODEL_HASH,
+                extract=True, cache_subdir='models'
             )
-
-            model.load_weights(weights_path, by_name=True)
-        else:
-            weights_path = None
+            model_path = os.path.splitext(archive_path)[0]
+            model = tf.keras.models.load_model(model_path)
 
         super(CytoplasmSegmentation, self).__init__(
             model,
-            model_image_shape=model_image_shape,
+            model_image_shape=model.input_shape[1:],
             model_mpp=0.65,
-            preprocessing_fn=phase_preprocess,
-            postprocessing_fn=deep_watershed,
+            preprocessing_fn=preprocessing_fn,
+            postprocessing_fn=postprocessing_fn,
             dataset_metadata=self.dataset_metadata,
             model_metadata=self.model_metadata)
 
@@ -133,8 +119,9 @@ class CytoplasmSegmentation(Application):
                 image,
                 batch_size=4,
                 image_mpp=None,
-                preprocess_kwargs={},
-                postprocess_kwargs={}):
+                pad_mode='reflect',
+                preprocess_kwargs=None,
+                postprocess_kwargs=None):
         """Generates a labeled image of the input running prediction with
         appropriate pre and post processing functions.
 
@@ -148,6 +135,7 @@ class CytoplasmSegmentation(Application):
                 ``[batch, x, y, channel]``.
             batch_size (int): Number of images to predict on per batch.
             image_mpp (float): Microns per pixel for ``image``.
+            pad_mode (str): The padding mode, one of "constant" or "reflect".
             preprocess_kwargs (dict): Keyword arguments to pass to the
                 pre-processing function.
             postprocess_kwargs (dict): Keyword arguments to pass to the
@@ -163,9 +151,22 @@ class CytoplasmSegmentation(Application):
         Returns:
             numpy.array: Labeled image
         """
+        if preprocess_kwargs is None:
+            preprocess_kwargs = {}
+
+        if postprocess_kwargs is None:
+            postprocess_kwargs = {
+                'radius': 10,
+                'maxima_threshold': 0.1,
+                'interior_threshold': 0.01,
+                'exclude_border': False,
+                'small_objects_threshold': 0
+            }
+
         return self._predict_segmentation(
             image,
             batch_size=batch_size,
             image_mpp=image_mpp,
+            pad_mode=pad_mode,
             preprocess_kwargs=preprocess_kwargs,
             postprocess_kwargs=postprocess_kwargs)
