@@ -1,4 +1,4 @@
-# Copyright 2016-2020 The Van Valen Lab at the California Institute of
+# Copyright 2016-2021 The Van Valen Lab at the California Institute of
 # Technology (Caltech), with support from the Paul Allen Family Foundation,
 # Google, & National Institutes of Health (NIH) under Grant U24CA224309-01.
 # All rights reserved.
@@ -40,7 +40,8 @@ from deepcell.applications import Application
 
 
 MODEL_PATH = ('https://deepcell-data.s3-us-west-1.amazonaws.com/'
-              'saved-models/NuclearSegmentation-3.tar.gz')
+              'saved-models/NuclearSegmentation-5.tar.gz')
+MODEL_HASH = 'c3667ffbe08035c9cb69ed882e4a49d7'
 
 
 class NuclearSegmentation(Application):
@@ -92,12 +93,14 @@ class NuclearSegmentation(Application):
         'validation_steps_per_epoch': 15627
     }
 
-    def __init__(self, model=None):
+    def __init__(self, model=None,
+                 preprocessing_fn=histogram_normalization,
+                 postprocessing_fn=deep_watershed):
 
         if model is None:
             archive_path = tf.keras.utils.get_file(
                 'NuclearSegmentation.tgz', MODEL_PATH,
-                file_hash='7fff56a59f453252f24967cfe1813abd',
+                file_hash=MODEL_HASH,
                 extract=True, cache_subdir='models'
             )
             model_path = os.path.splitext(archive_path)[0]
@@ -107,8 +110,8 @@ class NuclearSegmentation(Application):
             model,
             model_image_shape=model.input_shape[1:],
             model_mpp=0.65,
-            preprocessing_fn=histogram_normalization,
-            postprocessing_fn=deep_watershed,
+            preprocessing_fn=preprocessing_fn,
+            postprocessing_fn=postprocessing_fn,
             dataset_metadata=self.dataset_metadata,
             model_metadata=self.model_metadata)
 
@@ -149,15 +152,13 @@ class NuclearSegmentation(Application):
             numpy.array: Labeled image
         """
         if preprocess_kwargs is None:
-            preprocess_kwargs = {
-                'kernel_size': 64
-            }
+            preprocess_kwargs = {}
 
         if postprocess_kwargs is None:
             postprocess_kwargs = {
-                'min_distance': 10,
-                'detection_threshold': 0.1,
-                'distance_threshold': 0.01,
+                'radius': 10,
+                'maxima_threshold': 0.1,
+                'interior_threshold': 0.01,
                 'exclude_border': False,
                 'small_objects_threshold': 0
             }
